/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.node;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.util.SortedList;

/**
 * Utility functions for working with AuthorNodes.
 * 
 * @author alex_jitianu
 *
 * @since 15.2
 * 
 * <br>
 * <br>
 * *********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * ********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the 
 * next versions of the application. If you have suggestions, 
 * comments about it, please let us know.</p> 
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public class AuthorNodeUtil {
  /**
   * Remove from the collection of nodes any descendants so the only nodes that are 
   * kept are disjunct.
   * 
   * @param collection A collection of nodes.
   * 
   * @return A list of disjunct nodes.
   */
  public static List<AuthorNode> minimizeAuthorCollection(Collection<AuthorNode> collection) {
    List<AuthorNode> list = new SortedList<AuthorNode>(new Comparator<AuthorNode>() {
      @Override
      public int compare(AuthorNode o1, AuthorNode o2) {
        return o1.getStartOffset() - o2.getStartOffset();
      }
    });
    
    // Filter any null references.
    for (Iterator iterator = collection.iterator(); iterator.hasNext();) {
      AuthorNode authorNode = (AuthorNode) iterator.next();
      if (authorNode != null) {
        if (authorNode.getOwnerDocument() == null) {
          // This is a removed node. Clean it up.
          iterator.remove();
        } else {
          list.add(authorNode);
        }
      }
    }
    
    List<AuthorNode> minimized = new ArrayList<AuthorNode>();
    for (int i = 0; i < list.size(); i++) {
      AuthorNode current = list.get(i);
      AuthorNode previous = minimized.isEmpty() ? null : minimized.get(minimized.size() - 1);
      if (previous != null 
          && previous.getStartOffset() < current.getStartOffset() 
          && current.getEndOffset() < previous.getEndOffset()) {
        // The current node is contained in the previous one.
      } else {
        minimized.add(current);
      }
    }
    
    return minimized;
  }
}
