/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.webapp;

import java.io.IOException;

import javax.swing.text.BadLocationException;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * A class that renders the author document or parts of it in an HTML format.
 * 
 * Each XML node is rendered as an HTML 'div' that contains:
 * - the start sentiel (and possibly the setinels of the parents in compact view
 *   mode) 
 * - the static content of the ':before' child specified in CSS
 * - the static content of the node specified in CSS.
 * - The 'div's that correspond to the children of the node.
 * - Spans of text between every two consecutive children (possibly empty).
 * - the static content of the ':after' child specified in CSS
 * - the end sentinel(s)
 *
 * @since 15.1
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface AuthorHTMLRenderer {

  /**
   * Delimiter char between the tag name and namespace used to encode both of
   * them in a single attribute.
   */
  public static final String TAG_NAME_DELIMITER = "!";
  

  /**
   * Renders the entire document in HTML format.
   * 
   * @throws BadLocationException
   * @throws IOException
   */
  public abstract void render() throws BadLocationException, IOException;

  /**
   * Renders only the content and sentinels of the document (without the 'div'
   * HTML tag).
   * 
   * @throws BadLocationException
   * @throws IOException
   */
  public abstract void renderContentAndSentinels() throws BadLocationException, IOException;

  /**
   * Renders the content and sentinels of a specific node from the document 
   * (without the 'div' HTML tag). 
   *
   * @param node The node to be rendered.
   * 
   * @throws BadLocationException
   * @throws IOException
   */
  public abstract void renderNodeContentAndSentinels(AuthorNode node) throws BadLocationException,
      IOException;
  
  /**
   * Renders the content of the node including the 'div' HTML tag.
   * 
   * @param node The author node to render.
   * 
   * @throws BadLocationException
   * @throws IOException
   */
  public abstract void renderNode(AuthorNode node) throws BadLocationException, IOException;

}