package ro.sync.ecss.extensions.api.webapp;

import java.io.File;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.w3c.css.sac.InputSource;
import org.xml.sax.SAXException;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.document.Descriptor;
import ro.sync.ecss.AuthorErrorHandler;
import ro.sync.ecss.component.AuthorCCManagerProvider;
import ro.sync.ecss.component.AuthorContentReferencesResolverImpl;
import ro.sync.ecss.component.CSSInputSource;
import ro.sync.ecss.component.ControllerExtensionsAccess;
import ro.sync.ecss.component.IndentOptionsProvider;
import ro.sync.ecss.contentcompletion.AuthorCCManager;
import ro.sync.ecss.css.CssRulesList;
import ro.sync.ecss.css.IStyleSheet;
import ro.sync.ecss.css.ParsedContent;
import ro.sync.ecss.css.StylesheetFactory;
import ro.sync.ecss.dom.AuthorDocumentImpl;
import ro.sync.ecss.dom.builder.AuthorDocumentFactory;
import ro.sync.ecss.dom.builder.AuthorSpacesNormalizer;
import ro.sync.ecss.dom.builder.StylesheetException;
import ro.sync.ecss.dom.builder.states.AuthorElementContentInformationProvider;
import ro.sync.ecss.extensions.CSSResource;
import ro.sync.ecss.extensions.MergedCSSs;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorAccessProvider;
import ro.sync.ecss.extensions.api.AuthorExtensionActionProvider;
import ro.sync.ecss.extensions.api.AuthorExtensionStateListener;
import ro.sync.ecss.extensions.api.AuthorReferenceResolver;
import ro.sync.ecss.extensions.api.AuthorSelectionAndCaretModel;
import ro.sync.ecss.extensions.api.ExtensionsBundle;
import ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.webapp.cc.ContentCompletionManager;
import ro.sync.ecss.persistance.CSSDescriptorPO;
import ro.sync.ecss.strictediting.DefaultAuthorSchemaAwareEditingSupport;
import ro.sync.ecss.ue.AuthorDocumentControllerImpl;
import ro.sync.ecss.webapp.AuthorDocumentModelImpl;
import ro.sync.ecss.webapp.AuthorIdIndexImpl;
import ro.sync.ecss.webapp.AuthorMarkListener;
import ro.sync.ecss.webapp.SimpleSelectionAndCaretModel;
import ro.sync.ecss.webapp.WebappAuthorSchemaAwareActionsSupport;
import ro.sync.ecss.webapp.WebappHeadlessViewport;
import ro.sync.ecss.webapp.WebappIDEAccess;
import ro.sync.ecss.webapp.access.WebappAuthorAccess;
import ro.sync.ecss.webapp.actions.AuthorActionsManagerImpl;
import ro.sync.ecss.webapp.actions.AuthorXpathEffector;
import ro.sync.ecss.webapp.cc.ContentCompletionManagerImpl;
import ro.sync.ecss.webapp.css.CSSTranslator;
import ro.sync.ecss.webapp.css.FilterCSSRuleList;
import ro.sync.ecss.webapp.css.FontManager;
import ro.sync.ecss.webapp.formcontrols.WebappInplaceEditingHelper;
import ro.sync.ecss.webapp.review.ReviewControllerImpl;
import ro.sync.exml.IDEAccess;
import ro.sync.exml.OxygenSystemProperties;
import ro.sync.exml.editor.ContentTypes;
import ro.sync.exml.editor.EditorDocumentType;
import ro.sync.exml.editor.xmleditor.DocumentTypeProvider;
import ro.sync.exml.editor.xmleditor.prettyprint.IndentOptions;
import ro.sync.exml.editor.xmleditor.prettyprint.PrettyPrintAnalyser;
import ro.sync.exml.editor.xsdeditor.XMLSchemaVersion;
import ro.sync.exml.license.LicenseChecker;
import ro.sync.exml.license.LicenseServerInformation;
import ro.sync.exml.license.LicenseServerInformation.LicenseServerType;
import ro.sync.exml.options.LicenseOptions;
import ro.sync.exml.options.OptionConstants;
import ro.sync.exml.options.OptionTags;
import ro.sync.exml.options.Options;
import ro.sync.exml.schema.ExternalDTDInfo;
import ro.sync.exml.serialize.state.SchemaContentInfoProviderCache;
import ro.sync.exml.zip.ArchiveUtilAdaptor;
import ro.sync.exml.zip.ArchiveUtilFacade;
import ro.sync.io.IOUtil;
import ro.sync.io.NonCloseableReader;
import ro.sync.io.NonCloseableReaderWrapper;
import ro.sync.io.RewindableReader;
import ro.sync.licensemanager.FloatingLicenseCallback;
import ro.sync.licensemanager.LicenseInformation;
import ro.sync.licensemanager.LicenseInformationProvider;
import ro.sync.net.protocol.Installer;
import ro.sync.ui.PPAnalysedEditor;
import ro.sync.ui.application.bundle.BundleDescriptor;
import ro.sync.util.PreferencesDirectoryDetector;
import ro.sync.util.SystemIdProvider;
import ro.sync.util.URLUtil;
import ro.sync.xml.encoding.EncodingDetector;
import ro.sync.xml.xpath.InContextXPathEffector;

/**
 * Factory class that creates the document model to be used in the Web Reviewer.
 * 
 * @since 15.1
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public class WebappAuthorDocumentFactory {

	/**
	 * Logger for logging.
	 */
	static final Logger logger = Logger
			.getLogger(AuthorDocumentFactory.class.getName());

	/**
   * Factory method that creates a document model for the document with the 
   * specified systemID.
   *
   * It takes care of detecting the encoding, resolving DTD, CSS resources, etc.
   * 
   * @param systemIdUrl The document URL.
   * @param docReader Reader over the document.
   * @param bomBytes BOM bytes.
	 * @param sessionAttributes A map of session attributes.
   * 
   * @return The author document.
   * 
   * @throws IOException 
   * @throws ParserConfigurationException 
   * @throws SAXException 
   */
	public static AuthorDocumentModel createAuthorDocumentInfo(URL systemIdUrl, 
	    Reader docReader, List bomBytes, Map<String, ?> sessionAttributes) 
	    throws IOException, ParserConfigurationException, SAXException {

    init();

    // A rewindable reader over the document.
    RewindableReader rr = new RewindableReader(docReader);

    // The document type provider.
    DocumentTypeProvider documentTypeProvider = getDocumentTypeProvider(systemIdUrl, rr);

    // Editor document type.
    EditorDocumentType docType = documentTypeProvider.getEditorDocumentType();
    
    // Create the author error handler.
    AuthorErrorHandler errorhandler = createErrorHandler(systemIdUrl);
    
    // Instantiate the font manager.
    FontManager fontManager = new FontManager();
    
    CSSInputSource[] cssInputSources = new CSSInputSource[0];
    
    CssRulesList complexCSSRules = null;
    
    String css = "";
    
    if (docType.getCSSDescriptors() != null && docType.getCSSDescriptors().length != 0) {
      // Get the CSS to be used.
      List<CSSResource> cssUrls = getCSSsToUse(
          docType.getCSSDescriptors(), systemIdUrl.toExternalForm());


      // Parse the CSS files.
      ParsedContent rules = StylesheetFactory.getParsedContent(
          createInputSources(cssUrls).toArray(new CSSInputSource[0]), 
          errorhandler, 
          false);

      // Generate the CSS rules to apply to the HTML page that are simple enough
      // to be understood by the browsers.
      String simpleCssContent = generateSimpleCSSContent(rules);

      // The CSS to be applied to the HTML document including font-face declarations.
      css = 
          "." + HTMLClasses.DOCUMENT + " { font-family: " + fontManager.getHostFont().getName() + ";}" +
          simpleCssContent;
      
      // Create CSS input sources.
      cssInputSources = createInputSources(cssUrls).toArray(new CSSInputSource[0]);
      // Compute the list of complex CSS rules.
      complexCSSRules = new FilterCSSRuleList(rules).getCSSRuleList();
    } else {
      css = "." + HTMLClasses.DOCUMENT + ":before {color: red; content: \"Cannot load the associated CSS files.\"}";
      CSSInputSource cssInputSource = new CSSInputSource(
          new InputSource(new StringReader( "* {\n" + 
              "    display:block;\n" + 
              "}")), 
              IStyleSheet.SOURCE_DOCUMENT_TYPE);
      cssInputSources = new CSSInputSource[] {cssInputSource}; 
      complexCSSRules = new CssRulesList();
    }

    // Obtain the external DTD system ID.
    ExternalDTDInfo dtdSystemID = DocumentTypeProvider.getAssociatedDTDURL(docType);

    // Element states provider for handling whitespaces.
    AuthorElementContentInformationProvider elementStatesProvider = 
        createElementStatesProvider(systemIdUrl, rr, documentTypeProvider);

    // The author XML document model.
    rr.rewind();
    AuthorDocumentImpl xmlDoc = null;
    try {
      xmlDoc = AuthorDocumentFactory.create(
          rr,
          systemIdUrl.toExternalForm(), 
          cssInputSources, 
          errorhandler,
          fontManager, 
          null /** Styles filter */, 
          null /** Conditions styles filter */, 
          dtdSystemID, 
          elementStatesProvider, 
          true);
    } catch (StylesheetException e) {
      throw new IOException(e);
    }
    
    // Append the user-delcared font-faces. 
    css += fontManager.getFontFaceCSSDeclarations();
    
    // Instantiate a selection model for the document.
    AuthorSelectionAndCaretModel selectionModel = new SimpleSelectionAndCaretModel();

    // The author content completion manager.
    final AuthorCCManager authorCCManager = new AuthorCCManager(documentTypeProvider);
    
    InContextXPathEffector xpathEffector = 
        new AuthorXpathEffector(createDocumentDescriptor(systemIdUrl, rr), xmlDoc, selectionModel);
    
    final ExtensionsBundle extensionsBundle = docType.getExtensionsBundle();
    
    WebappHeadlessViewport headlessViewport = new WebappHeadlessViewport(
        authorCCManager, 
        extensionsBundle, 
        selectionModel);
    
    // Instantiate the controller and register the model listener.
    final AuthorDocumentControllerImpl authorDocumentController = new AuthorDocumentControllerImpl(
        errorhandler, 100, headlessViewport, new AuthorCCManagerProvider() {
          @Override
          public AuthorCCManager getCCManager() {
            return authorCCManager;
          }
        }, new ControllerExtensionsAccess() {
          
          @Override
          public ExtensionsBundle getAuthorExtensionsBundle() {
            return extensionsBundle;
          }
        }, xpathEffector, // xpath effector
        true,
        // We'll set it later on when we can finally build one.
        null,
        documentTypeProvider);
    authorDocumentController.setDocument(xmlDoc);
    authorDocumentController.setMarkListener(new AuthorMarkListener(selectionModel));
    headlessViewport.setController(authorDocumentController);
    
    // Instantiate the review controller.
    ReviewControllerImpl reviewController = new ReviewControllerImpl(authorDocumentController);
    headlessViewport.setChangeTrackingController(reviewController);

    // Learn the indent options of the document.
    final IndentOptions indentOptions = learnIndentOptions(rr);

    // Instantiate the content completion manager.
    ContentCompletionManager contentCompletionManager = new ContentCompletionManagerImpl(
        authorDocumentController, 
        authorCCManager);

    // Instantiate the markers indexer.
    AuthorIdIndex<AuthorPersistentHighlight> markersIndexer = new AuthorIdIndexImpl<AuthorPersistentHighlight>();

    // Instantiate the nodes indexer.
    AuthorIdIndex<AuthorNode> nodesIndexer = new AuthorIdIndexImpl<AuthorNode>();

    // The encoding of the document.
    String javaEncoding = EncodingDetector.getJavaEncodingFromHeader(
        createNonCloseableReader(rr), bomBytes);
    if (javaEncoding == null) {
      javaEncoding = "UTF-8";
    }

    final AuthorAccess authorAccess = 
        new WebappAuthorAccess(
            authorDocumentController, 
            documentTypeProvider, 
            extensionsBundle, 
            selectionModel, 
            headlessViewport,
            new IndentOptionsProvider() {
              @Override
              public IndentOptions getIndentOptions() {
                return indentOptions;
              }
            },
            sessionAttributes);
    
    // Pass the author access to the author document controller.
    authorDocumentController.setAuthorAccess(new AuthorAccessProvider() {
      @Override
      public AuthorAccess getAuthorAccess() {
        return authorAccess;
      }
    });    
    
    // Initialize the link text resolver.
    headlessViewport.intializeLinkResolver(authorAccess);

    
    AuthorExtensionActionProvider actionsManager = new AuthorActionsManagerImpl(
        authorAccess, 
        docType.getID(), 
        docType.getAuthorExtensionPO(), 
        docType.getClasspath(),
        xpathEffector,
        selectionModel,
        authorDocumentController);
    
    WebappInplaceEditingHelper editingHelper = new WebappInplaceEditingHelper(
        authorDocumentController, authorAccess, actionsManager);

    // Actions support.
    WebappAuthorSchemaAwareActionsSupport actionsSupport = new WebappAuthorSchemaAwareActionsSupport(
            authorDocumentController, 
            selectionModel, 
            authorAccess, 
            extensionsBundle, 
            errorhandler, 
            new DefaultAuthorSchemaAwareEditingSupport(headlessViewport));
    // Instantiate the review controller.
    reviewController.setActionsSupport(actionsSupport);
    headlessViewport.setActionsSupport(actionsSupport);

    AuthorContentReferencesResolverImpl referencesResolver = 
        new AuthorContentReferencesResolverImpl(errorhandler, authorAccess);

    if (extensionsBundle != null) {
      // Pass the author access to the extensions bundle.
      AuthorExtensionStateListener authorExtensionStateListener = 
          extensionsBundle.createAuthorExtensionStateListener();
      if (authorExtensionStateListener != null) {
        authorExtensionStateListener.activated(authorAccess);
      }
      
      // instantiate the reference resolver.
      AuthorReferenceResolver referenceResolver = extensionsBundle.createAuthorReferenceResolver();
      if (referenceResolver != null) {
        referencesResolver.addReferenceResolver(referenceResolver);
      }
    }
    
    // Resolve references in the docuement.
    authorDocumentController.setReferencesResolver(referencesResolver);
    authorDocumentController.resolveDocumentReferences(xmlDoc, false);

    return new AuthorDocumentModelImpl(xmlDoc, 
        authorDocumentController,
        documentTypeProvider, 
        indentOptions, 
        css, 
        complexCSSRules, 
        contentCompletionManager,
        reviewController,
        markersIndexer,
        nodesIndexer,
        selectionModel,
        editingHelper,
        actionsManager,
        javaEncoding,
        actionsSupport,
        authorAccess);
  
	}
	
	/**
	 * Factory method that creates a document model for the document with the 
	 * specified systemID.
	 *
	 * It takes care of detecting the encoding, resolving DTD, CSS resources, etc.
	 * 
	 * @param systemID The document systemID.
	 * @param sessionAttributes The session attributes.
	 * 
	 * @return The author document.
	 * 
	 * @throws IOException 
	 * @throws ParserConfigurationException 
	 * @throws SAXException 
	 */
	public static AuthorDocumentModel createAuthorDocumentInfo(String systemID, 
	    Map<String, ?> sessionAttributes) throws IOException, 
	    ParserConfigurationException, SAXException {
	  URL systemIdUrl = new URL(systemID);
	  
	// The reader over the document that detects the encoding.
    @SuppressWarnings("rawtypes")
    List bomBytes = new ArrayList();
    Reader docReader = EncodingDetector.createReader(systemIdUrl, null, "UTF8",
        bomBytes);
    
    return createAuthorDocumentInfo(systemIdUrl, docReader, bomBytes, sessionAttributes);
	}
	
	/**
   * Factory method that creates a document model for the document with the 
   * specified systemID.
   *
   * It takes care of detecting the encoding, resolving DTD, CSS resources, etc.
   * 
   * @param systemID The document systemID.
   * @param docReader Reader over the document.
	 * @param sessionAttributes Session attributes.
   * 
   * @return The author document.
   * 
   * @throws IOException 
   * @throws ParserConfigurationException 
   * @throws SAXException 
   */
	public static AuthorDocumentModel createAuthorDocumentInfo(String systemID, 
	    Reader docReader, Map<String, ?> sessionAttributes)
      throws IOException, ParserConfigurationException, SAXException {
    URL systemIdUrl = new URL(systemID);
    
    // The reader over the document that detects the encoding.
    @SuppressWarnings("rawtypes")
    List bomBytes = new ArrayList();
    
    return createAuthorDocumentInfo(systemIdUrl, docReader, bomBytes, sessionAttributes);
  }

	/** 
	 * Generate the CSS to be applied to the HTML document for the CSS rules
	 * that are simple enough to be understood by the browser.
	 * 
	 * @param rules The parsed CSS rules.
	 * 
	 * @return A string with the CSS content.
	 */
  private static String generateSimpleCSSContent(ParsedContent rules) {
    Writer out = new StringWriter();
		CSSTranslator translator = new CSSTranslator();
		translator.serialize(out, rules);
		String cssCotnent = out.toString();
    return cssCotnent;
  }

  /**
   * Returns the document type provider for the document with the given URL.
   * 
   * @param xmlURL The URL of the document.
   * @param rr A rewindable reader over the document.
   * 
   * @return The document type provider which already detected the document type.
   */
  private static DocumentTypeProvider getDocumentTypeProvider(final URL xmlURL, final RewindableReader rr) {
    Descriptor documentDescriptor = createDocumentDescriptor(xmlURL, rr);
    DocumentTypeProvider documentTypeProvider = new DocumentTypeProvider(documentDescriptor);
		documentTypeProvider.detectNow(true);
    return documentTypeProvider;
  }

  /**
   * Creates an error handler for the document with the given URL.
   * 
   * @param xmlURL The URL of the document.
   * 
   * @return An author error handler.
   */
  private static AuthorErrorHandler createErrorHandler(final URL xmlURL) {
    // TODO (EXM-28704) maybe add error listener to error handler
    AuthorErrorHandler errorhandler = new AuthorErrorHandler(
				new SystemIdProvider() {
					@Override
					public String getSystemId() {
						return xmlURL.toExternalForm();
					}
				});
    return errorhandler;
  }

  /**
   * Creates an element states provider for the document with the specified URL.
   * 
   * @param xmlURL The URL of the document.
   * @param rr A rewindable reader over the document.
   * @param editorDocumentType The type of the document. 
   * 
   * @return An element states provider for the document with the specified URL.
   */
  private static AuthorElementContentInformationProvider createElementStatesProvider(URL xmlURL,
      RewindableReader rr, DocumentTypeProvider documentTypeProvider) {
    return AuthorSpacesNormalizer
    .createElementStatesProvider(SchemaContentInfoProviderCache
        .getInstance().getContentInformation(
            xmlURL,
            documentTypeProvider.getEditorDocumentType().getSchemaDescriptors(),
            documentTypeProvider.getLastAnalyzedHeader()));
  }

  /**
   * Creates a document descriptor for the document with the given URL.
   * 
   * @param xmlURL The URL of the document.
   * @param rr A rewindable reader over the document.
   * 
   * @return The document descriptor.
   */
  private static Descriptor createDocumentDescriptor(final URL xmlURL, final RewindableReader rr) {
    Descriptor documentDescriptor = new Descriptor() {
			@Override
			public URL getEditorLocation() {
				return xmlURL;
			}
		
			@Override
			public String getContentType() {
				return ContentTypes.XML_CONTENT_TYPE;
			}
		
			@Override
			public NonCloseableReader createReader() {
			  return createNonCloseableReader(rr);
			}
		};
    return documentDescriptor;
  }

  /**
   * Create input sources for the files with the given CSS URLs.
   * 
   * @param cssUrl The URLs of the CSS files.
   * 
   * @return The input sources.
   */
	private static List<CSSInputSource> createInputSources(List<CSSResource> cssUrls) {
	  List<CSSInputSource> cssInputSources = new ArrayList<CSSInputSource>();
		if (cssUrls != null && cssUrls.size() > 0) {
	    for (CSSResource cssUrl : cssUrls) {
	      cssInputSources.add(new CSSInputSource(
	            cssUrl.createInputSource(), cssUrl.getOrigin()));
	    }
			
		}
		return cssInputSources;
	}
	
	/**
	 * Learns the indentation options for the XML content of the reader.
	 * 
	 * @param rr A reader over the XML document.
	 * 
	 * @return The indentation options.
	 */
	private static IndentOptions learnIndentOptions(final RewindableReader rr) {
    rr.rewind();
    IndentOptions indentOptions = PrettyPrintAnalyser.getIndentOptions(new PPAnalysedEditor() {
      @Override
      public PrettyPrintAnalyser getPrettyPrintAnalyser() {
        PrettyPrintAnalyser prettyPrintAnalyser = new PrettyPrintAnalyser();
        // Perform the indent options analysis.
        try {
          int indentSize = Options.getInstance().getIntegerProperty(Options.EDITOR_INDENT_SIZE);
          int lineWidth = Options.getInstance().getIntegerProperty(Options.EDITOR_LINE_WIDTH);
          IOUtil.read(prettyPrintAnalyser.getAnalyzerFilter(rr, indentSize, lineWidth));
        } catch (IOException e) {
          throw new AssertionError();
        }
        return prettyPrintAnalyser;
      }
    });
    return indentOptions;
	}

	/**
   * Returns a list of CSS URLs to be used.
   * 
   * @param cssDescriptors The css descriptors.
   * @return The list of URLs.
   */
  private static List<CSSResource> getCSSsToUse(CSSDescriptorPO[] cssDescriptors, String systemId) {
    List<CSSResource> cssUrls = new ArrayList<CSSResource>();
    // Get the css's to be used from the descriptor.
    // Merge the CSS's.
    List<MergedCSSs> mergedCSSsList = MergedCSSs.getMergedCSSs(cssDescriptors);
    MergedCSSs cssToUse = mergedCSSsList.get(0);
    if (cssToUse != null) {
      cssUrls = resolveCSSURLs(cssToUse, systemId);
    }
    return cssUrls;
  }
	  
  /**
   * Expand editor variables and correct CSS URLs.
   * 
   * @param mergedCSSs The merged css to be analyzed.
   * @return An array of URLs representing the CSSs to be used.
   */
  private static List<CSSResource> resolveCSSURLs(MergedCSSs mergedCSSs, String systemId) {
    List<CSSResource> urls = new ArrayList<CSSResource>();
    for (CSSResource cssResource : mergedCSSs.getUrls()) {
      try {
        urls.add(cssResource.resolve(systemId));
      } catch (MalformedURLException ex) {
        logger.error(ex, ex);
      }
    }
    return urls;
  }

  /**
   * Creates a non-closeable reader over the specified rewindable reader.
   * 
   * @param rr The rewindable reader.
   * 
   * @return A non-closeable reader.
   */
  private static NonCloseableReader createNonCloseableReader(final RewindableReader rr) {
    return new NonCloseableReaderWrapper() {
      @Override
      public Reader createWrappedReader() throws IOException {
        // Rewind the reader
        rr.rewind();
        return rr;
      }
    };
  }
  

  /**
   * Sets the file to be used to load the oxygen options.
   * 
   * This method should be called before any other use of the Options singleton.
   * 
   * @param workDir The directory in which the options file resides.
   * @param optionsFileName The name of the options file.
   */
  public static void setOptions(File workDir, String optionsFileName) {
    // Set the name of preferences directory from application descriptor.
    //This is the first thing we MUST do.
    PreferencesDirectoryDetector.setPrefsDir(workDir.getAbsolutePath());
    Options.setFixedOptionsFileName(optionsFileName);
    init();
    setDefaultOptions();
  }
  
  /**
   * Sets the folder that contains the frameworks.
   * 
   * @param workDir The folder that contains the frameworks.
   */
  public static void setFrameworks(File workDir) {
    init();
    
    Options.getInstance().setBooleanProperty(
        OptionTags.USE_CUSTOM_FRAMEWORKS_DIRECTORY_URL,
        true);

    // Set the new frameworks directory.
    try {
      Options.getInstance().setStringProperty(
          OptionTags.FRAMEWORKS_DIRECTORY_URL_WITH_EDITOR_VARIABLES,
          URLUtil.correct(workDir).toString());
    } catch (MalformedURLException e) {
      logger.error(e, e);
    }
  }    
  
  
  /**
   * Sets the license server for the floating licensing.
   * 
   * @param servletURL The URL of the license server. 
   * @param userName The user name.
   * @param password The password.
   */
  public static void setLicenseServer(String servletURL, String userName, String password) {
    init();
    LicenseServerInformation serverInformation = new LicenseServerInformation(
        LicenseServerType.HTTP_SERVER, null, -1, servletURL, userName, password);
    LicenseOptions.getInstance().setLicenseServerInformation(serverInformation);
    checkLicense(null);
  }

  /**
   * Sets the license key.
   * 
   * @param licenseKey The licence key. 
   */
  public static void setLicenseKey(String licenseKey) {
    init();
    //User based
    LicenseOptions.getInstance().setLicenseText(licenseKey);

    checkLicense(licenseKey);
  }

  /**
   * Checks whether the webapp has the required licence.
   * 
   * @param licenseKey The licence key, or null if a floating licence is used.
   */
  private static void checkLicense(String licenseKey) {
    // Create the license checker.
    final BundleDescriptor BUNDLE_DESCRIPTOR = null;
    LicenseChecker licenseChecker = new LicenseChecker(null, null, BUNDLE_DESCRIPTOR, new FloatingLicenseCallback() {
      @Override
      public void showErrorMessage(IOException ex) {
        // Do nothing.
        // TODO (EXM-28705)
      }

      @Override
      public void licenseViolated(int maximum, int count, String connectedUsers) {
        // TODO (EXM-28705)
      }

      @Override
      public void licenseChanged() {
        // Ignore.
      }

    }) {
      /**
       * @see ro.sync.exml.license.AbstractLicenseChecker#loadLicenseTextFromFile()
       */
      @Override
      protected String loadLicenseTextFromFile() {
        //No fixed license key to be loaded from text file.
        return null;
      }
      /**
       * @see ro.sync.exml.license.LicenseChecker#displayLicenseVersionGreaterThanProduct(java.lang.String)
       */
      @Override
      protected void displayLicenseVersionGreaterThanProduct(String message) {
        //EXM-22402 Ignore this warning, the applet may block.
      }
    };
    //Check the license, fail if not available.
    LicenseInformation licenseInfo = licenseChecker.licenseCheck(true); 
    if (licenseInfo  == null) {
      // TODO (EXM-28705)
    } else if(!licenseInfo.isEvaluationOnly()) {
      if(licenseKey != null) {
        //A fixed license key was injected
        //EXM-22115
        //Not for evaluation, we accept only site license as an alternative
        if(licenseInfo.getCategory() != null && licenseInfo.getCategory().endsWith("-Site")){
          //Good.
        } else {
          //Bad.
          // TODO (EXM-28705)
        }
      } else {
        //The user was asked for a license key, any key is accepted.
      }
    }

    //EXM-22115 Set the license information
    LicenseInformationProvider.getInstance().setLicenseInformation(licenseInfo);
  }
  
  /**
   * Set some default options
   */
  private static void setDefaultOptions() {
    //XML Schema 1.0 for now, 1.1 requires additional JARs
    Options.getInstance().setStringProperty(OptionTags.DEFAULT_XML_SCHEMA_VERSION, XMLSchemaVersion.VERSION_1_0.getVersionID());
    Options.getInstance().setIntegerProperty(OptionTags.ASK_OPEN_DITA_MAP_IN_DITAMANAGER, OptionConstants.ALWAYS_NO);
  }

  /**
   * Flag indicating whether the web application has been already initialized.
   */
  private static boolean initialized = false;
  
  /**
   * Initialize the web application.
   */
  private static void init() {
    if (!initialized) {
      initialized = true;
      //Used in the bundle descriptor
      System.setProperty(OxygenSystemProperties.COM_OXYGENXML_BUNDLE_DESCRIPTOR, BundleDescriptor.WEBAPP_DESCRIPTOR_CONTENT);

      //Skips protocols
      Installer.inhibitInstallProtocols();

      IDEAccess.setInstance(new WebappIDEAccess());
      //No support for archives.
      ArchiveUtilFacade.setArchiveUtil(new ArchiveUtilAdaptor());
      //EXM-21610 API to control browse CMS from the applet.
    }
  }
}
