package ro.sync.ecss.extensions.api.webapp.review;

import java.util.List;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorChangeTrackingController;
import ro.sync.ecss.extensions.api.AuthorReviewerNameController;
import ro.sync.ecss.extensions.api.highlights.AuthorPersistentHighlight;

/**
 * Provides support for marker related actions (accept/reject tracked changes, add/edit comments, edit author name etc.).
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface ReviewController extends AuthorChangeTrackingController, AuthorReviewerNameController {

	/**
	 * Adds a comment marker at given offsets.
	 * 
	 * @param startOffset The start offset of the marker (inclusive).
	 * @param endOffset The end offset of the marker (exclusive).
	 * @param comment The comment of the marker.
	 */
	public abstract void addComment(int startOffset, int endOffset,
			String comment);

	/**
	 * Removes a comment marker.
	 * 
	 * @param highlight The comment marker.
	 */
	public abstract void removeComment(AuthorPersistentHighlight highlight);

	/**
	 * Edits the comment of a marker (comment and tracked change).
	 * 
	 * @param highlight The marker.
	 * @param newComment The new comment.
	 */
	public abstract void editComment(AuthorPersistentHighlight highlight,
			String newComment);

  
  /**
   * @return An array with all highlights.
   */
  public List<AuthorPersistentHighlight> getAllHighlights(); 
  
  
}