/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api.license;

import java.util.List;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;


/**
 * Provides information about how the current running Oxygen instance is licensed.
 * 
 * @since 12.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface LicenseInformationProvider extends LicenseCategories, LicenseComponents {
  
  /**
   * Get the license category. One of the {@link LicenseCategories} constants.
   * Can be <code>null</code> if no license is available.
   * 
   * @return the license category. One of the {@link LicenseCategories} constants.
   */
  String getCategory();
  
  /**
   * Check if the Oxygen application is permanently licensed or has a trial license.
   * 
   * @return <code>true</code> if the Oxygen application is permanently licensed or <code>false</code> if it has a trial license.
   */
  boolean isPermanent();
  
  /**
   * Get the components which are currently licensed.
   * For example { {@link LicenseComponents#EDITOR_COMPONENT_NAME}, {@link LicenseComponents#SAXON_SA_COMPONENT_NAME} }.
   * Can be <code>null</code> if no license is available.
   * 
   * @return the components which are currently licensed.
   *         For example { {@link LicenseComponents#EDITOR_COMPONENT_NAME}, {@link LicenseComponents#SAXON_SA_COMPONENT_NAME} }
   */
  List<String> getComponents();
  
  /**
   * Check if the current application is licensed or not.
   *
   * @return <code>true</code> if there is a license for the current application or
   *         <code>false</code> if there is no license key for the current application.
   */
  boolean isLicensed();
  
  /**
   * Check if the current application has exceeded the amount of floating licenses allowed on a server.
   * In this case the user will not be able to open any more files in Oxygen.
   * This situation can change while the application is running and will be reported a couple of minutes after the application has started.
   *    
   * @return <code>true</code> if the current application has exceeded the amount of floating licenses allowed on a server.
   */
  boolean isInViolationOfLicense();
}