/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api.options;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;

/**
 * Provides properties values for a Data source.
 * 
 * @since 14.1
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface DataSourceConnectionInfo {
  /**
   * Property ID for the driver name used to access the database. It can not be <code>null</code>.
   */
  String DRIVER_NAME = "driver.name";
  /**
   * Property ID for the host name to connect to. For relational databases it can be <code>null</code>.
   */
  String HOST_NAME = "host.name";
  /**
   * Property ID for the initial database from server to connect to. It can be <code>null</code>.
   */
  String INITIAL_DATABASE = "initial.database";
  /**
   * Property ID for the port used to connect to host. For relational databases can be included into the URL.
   */
  String PORT = "port";
  /**
   * Property ID for the unique name of the connection.
   */
  String CONNECTION_NAME = "connection.name";
  /**
   * Property ID for the URL used to connect to the database. It can be <code>null</code>.
   */
  String URL= "url";
  /**
   * Property ID for the WEBDAV URL. Some databases can be accessed using a WEBDAV URL.
   * It can be <code>null</code>.
   */
  String WEBDAV_URL = "webdav.url";
  
  /**
   * Used to return properties values for a data source connection.
   * 
   * @param   propertyId The property identifier.
   * @return  The value of the property identified by the propertyId. 
   *          The propertyId can be one of the Id constants defined in the {@link DataSourceConnectionInfo}.
   */
  Object getProperty(String propertyId);
}