/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.customprotocol;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import ro.sync.exml.plugin.urlstreamhandler.URLChooserPluginExtension2;
import ro.sync.exml.plugin.urlstreamhandler.URLChooserToolbarExtension;
import ro.sync.exml.workspace.api.standalone.StandalonePluginWorkspace;
import ro.sync.ui.Icons;

/**
 * Plugin extension - custom protocol chooser extension
 */
public class CustomProtocolChooserExtension implements URLChooserPluginExtension2, URLChooserToolbarExtension {
  /**
   * The custom protocol name.
   */
  private static final String CPROTO = "cproto";

  /**
   * The dialog used to select the URL
   */
  private static class URLChooserDialog extends JDialog {

    // The text field containing the URL of the file to be opened
    private final JTextArea selectionArea = new JTextArea();
    
    // The selected files of the dialog.
    List<URL> selectedURLs = new ArrayList<URL>();

    /**
     * Constructor for the dialog
     * @param jFrame The parent frame.
     */
    private URLChooserDialog(Frame parentFrame) {
      super(parentFrame);
      setModal(true);
      setTitle("Open Using Custom Protocol");
      setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
      GridBagLayout layout = new GridBagLayout();
      getContentPane().setLayout(layout);
      JLabel label = new JLabel("URL:");
      label.setPreferredSize(label.getMinimumSize());
      // The text field containing the URL of the file to be opened
      selectionArea.setText(CPROTO + ":/");
      // Set the caret at the end of text
      selectionArea.setCaretPosition("cproto:/".length());
      selectionArea.setPreferredSize(new Dimension(250, 100));
      selectionArea.setBorder(BorderFactory.createEtchedBorder());

      // Opens a file chooser dialog
      JButton open = new JButton("Browse");

      JButton ok = new JButton("Open");
      ok.setPreferredSize(new Dimension(80, 25));
      JButton cancel = new JButton("Cancel");
      cancel.setPreferredSize(new Dimension(80, 25));
      Container contentPane = getContentPane();

      // Add the components to the content pane
      GridBagConstraints constr = new GridBagConstraints();
      constr.gridx = 0;
      constr.gridy = 0;
      constr.weightx = 0;
      constr.gridwidth = 1;
      constr.anchor = GridBagConstraints.EAST;
      constr.insets = new Insets(3, 3, 3, 3);
      contentPane.add(label, constr);
      
      constr.anchor = GridBagConstraints.CENTER;
      constr.gridy = 1;
      constr.fill = GridBagConstraints.VERTICAL;
      contentPane.add(new JPanel(), constr);
      
      // The text field.
      constr.gridx = 1;
      constr.gridy = 0;
      constr.weightx = 1;
      constr.gridheight = 2;
      constr.gridwidth = 4;
      constr.fill = GridBagConstraints.BOTH;
      contentPane.add(selectionArea, constr);
      
      // The browse button.
      constr.gridx = 5;
      constr.gridy = 0;
      constr.weightx = 0;
      constr.gridheight = 1;
      constr.gridwidth = 1;
      constr.fill = GridBagConstraints.NONE;
      contentPane.add(open, constr);
      
      constr.gridy = 1;
      constr.fill = GridBagConstraints.VERTICAL;
      contentPane.add(new JPanel(), constr);
      
      // The OK button.
      constr.gridx = 0;
      constr.gridy = 2;
      constr.weightx = 0;
      constr.gridwidth = 1;      
      constr.fill = GridBagConstraints.NONE;
      contentPane.add(ok, constr);

      // The cancel button.
      constr.gridx = 5;
      constr.gridy = 2;
      constr.weightx = 0;
      constr.gridwidth = 1;      
      contentPane.add(cancel, constr);

      // Sets the automatic size.
      pack();

      // Centers to the screen.
      setLocationRelativeTo(null);
      
      // Action listener for the open button
      // Opens a file chooser dialog and sets the text in the field to the URL of the selected file
      open.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JFileChooser chooser = new JFileChooser();
          chooser.setMultiSelectionEnabled(true);
          int returnVal = chooser.showOpenDialog(URLChooserDialog.this);
          selectedURLs.clear();
          if (returnVal == JFileChooser.APPROVE_OPTION) {
            File[] selectedFiles = chooser.getSelectedFiles();
            StringBuffer selection = new StringBuffer();
            try {
              for (int i = 0; i < selectedFiles.length; i++) {
                String cprotoURL = CPROTO + ":" + selectedFiles[i].toURI().toURL().getPath();
                selectedURLs.add(new URL(cprotoURL));
                selection.append(cprotoURL).append("\n");
              }
              selectionArea.setText(selection.toString());
              selectionArea.requestFocus();
              // Set the caret at the end of text
              selectionArea.setCaretPosition(selectionArea.getText().length());
            } catch (MalformedURLException e1) {
              e1.printStackTrace();
            }
          }
        }
      });

      // Add listener for OK button
      // close the dialog
      ok.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          canceled = false;
          setVisible(false);
        }
      });
      // Add listener for Cancel button
      // Clear the selectionArea and close the dialog
      cancel.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          canceled = true;
          setVisible(false);
        }
      });
    }

    /**
     * Flag telling if the user canceled the operation. This
     * is done either by clicking the CANCEL button or by closing
     * the dialog from the titlebar.
     */
    private boolean canceled = true;
    
    /**
     * Shows the dialog and returns the URL from the text field
     */
    URL[] selectURLs() {
      // Assume the user cancels the operation.
      canceled = true;
      setVisible(true);
      if (canceled) {
        return null;
      } else {
        return selectedURLs.toArray(new URL[0]);
      }
    }
  }
  

  /**
   * @see ro.sync.exml.plugin.urlstreamhandler.URLChooserPluginExtension2#chooseURLs(ro.sync.exml.workspace.api.standalone.StandalonePluginWorkspace)
   */
  public URL[] chooseURLs(StandalonePluginWorkspace workspaceAccess) {
    // Create the dialog
    URLChooserDialog urlChooser = new URLChooserDialog((Frame)workspaceAccess.getParentFrame());

    // Show the dialog and return the string from the text field
    URL[] urls = urlChooser.selectURLs();
    return urls;
  }

  /**
   * @return A menu name.
   */
  public String getMenuName() {
    return "Open using Custom Protocol";
  }

  /**
   * @see ro.sync.exml.plugin.urlstreamhandler.URLChooserToolbarExtension#getToolbarIcon()
   */
  public Icon getToolbarIcon() {
    return Icons.getIcon(Icons.OPEN_CUSTOM_PROTOCOL_TOOLBAR_STRING);
  }

  /**
   * @see ro.sync.exml.plugin.urlstreamhandler.URLChooserToolbarExtension#getToolbarTooltip()
   */
  public String getToolbarTooltip() {
    return "Open with Custom Protocol";
  }
}