/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.workspace;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ro.sync.exml.plugin.option.OptionPagePluginExtension;
import ro.sync.exml.workspace.api.PluginWorkspace;

/**
 * Plugin option page extension Custom Workspace Access Plugin Extension.
 */
public class CustomWorkspaceAccessOptionPagePluginExtension extends OptionPagePluginExtension {

  /**
   * The option key describing the temporary location for the saved files. 
   */
  public static final String KEY_SAVE_TEMPORARY_FILES_LOCATION = "save.tmp.location";

  /**
   * The option key describing the default checkout location for the directory chooser.
   */
  public static final String KEY_DEFAULT_CHECKOUT_LOCATION = "default.checkout.location";
  
  /**
   * The text filed for the temporary location of the saved files.
   */
  private JTextField saveTmpLocationTextField;

  /**
   * The text field for the default checkout location of the directory chooser.
   */
  private JTextField defaultCheckoutLocationTextField;

  /**
   * @see ro.sync.exml.plugin.option.OptionPagePluginExtension#apply(ro.sync.exml.workspace.api.PluginWorkspace)
   */
  @Override
  public void apply(PluginWorkspace pluginWorkspace) {
    // Save the new locations in the option storage.
    pluginWorkspace.getOptionsStorage().setOption(KEY_SAVE_TEMPORARY_FILES_LOCATION,
        !"".equals(saveTmpLocationTextField.getText()) ? saveTmpLocationTextField.getText() : null);
    pluginWorkspace.getOptionsStorage().setOption(KEY_DEFAULT_CHECKOUT_LOCATION,
        !"".equals(defaultCheckoutLocationTextField.getText()) ? defaultCheckoutLocationTextField.getText() : null);
  }

  /**
   * @see ro.sync.exml.plugin.option.OptionPagePluginExtension#restoreDefaults()
   */
  @Override
  public void restoreDefaults() {
    // Reset the text fields values. Empty string is used to map the <null> default values of the options.
    saveTmpLocationTextField.setText("");
    defaultCheckoutLocationTextField.setText("");
  }

  /**
   * @see ro.sync.exml.plugin.option.OptionPagePluginExtension#getTitle()
   */
  @Override
  public String getTitle() {
    return "Workspace Access Sample Plugin";
  }

  /**
   * @see ro.sync.exml.plugin.option.OptionPagePluginExtension#init(ro.sync.exml.workspace.api.PluginWorkspace)
   */
  @Override
  public JComponent init(final PluginWorkspace pluginWorkspace) {
    GridBagConstraints c = new GridBagConstraints();
    JPanel panel = new JPanel(new GridBagLayout());
    JLabel saveTmpLocationLbl = new JLabel("Save location for temporary files:");
    c.gridx = 0;
    c.gridy = 0;
    c.weightx = 0;
    c.weighty = 0;
    c.anchor = GridBagConstraints.WEST;
    panel.add(saveTmpLocationLbl, c);
    
    saveTmpLocationTextField = new JTextField();
    c.gridx ++;
    c.weightx = 1;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.insets = new Insets(0, 5, 0, 5);
    panel.add(saveTmpLocationTextField, c);
    
    JButton chooseSaveTmpLocatioBtn = new JButton("Choose");
    c.gridx ++;
    c.weightx = 0;
    c.insets = new Insets(0, 0, 0, 0);
    c.fill = GridBagConstraints.NONE;
    panel.add(chooseSaveTmpLocatioBtn, c);
    
    chooseSaveTmpLocatioBtn.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        File dir = pluginWorkspace.chooseDirectory();
        if (dir != null) {
          saveTmpLocationTextField.setText(dir.getAbsolutePath());
        }
      }
    });
    
    c.gridx = 0;
    c.gridy ++;
    JLabel defaultCheckOutLocationLbl = new JLabel("Default checkout location:");
    panel.add(defaultCheckOutLocationLbl, c);
    
    defaultCheckoutLocationTextField = new JTextField();
    c.gridx ++;
    c.weightx = 1;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.insets = new Insets(0, 5, 0, 5);
    panel.add(defaultCheckoutLocationTextField, c);
    
    JButton chooseDefaultCheckOutLocatioBtn = new JButton("Choose");
    c.gridx ++;
    c.weightx = 0;
    c.insets = new Insets(0, 0, 0, 0);
    c.fill = GridBagConstraints.NONE;
    panel.add(chooseDefaultCheckOutLocatioBtn, c);
    
    chooseDefaultCheckOutLocatioBtn.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent e) {
        File dir = pluginWorkspace.chooseDirectory();
        if (dir != null) {
          defaultCheckoutLocationTextField.setText(dir.getAbsolutePath());
        }
      }
    });
    
    c.gridx = 0;
    c.gridy ++;
    c.gridwidth = 3;
    c.weightx = 1;
    c.weighty = 1;
    c.fill = GridBagConstraints.BOTH;
    panel.add(new JPanel(), c);
    
    String saveTmpLocation = pluginWorkspace.getOptionsStorage().getOption(
        KEY_SAVE_TEMPORARY_FILES_LOCATION, 
        null);
    
    String defaultCheckOutLocation = pluginWorkspace.getOptionsStorage().getOption(
        KEY_DEFAULT_CHECKOUT_LOCATION, 
        null);
    
    // Initialize the text fields with the stored options.
    saveTmpLocationTextField.setText(saveTmpLocation != null ? saveTmpLocation : "");
    defaultCheckoutLocationTextField.setText(defaultCheckOutLocation != null ? defaultCheckOutLocation : "");
    
    return panel;
  }
}
